//invoked only the first time the model is loaded, or upon reload

var geom,snd1,snd2,swheel_id ; //  fly77


function init_chassis()
{
  var wheelparam = {
    radius: 0.45,                      // fly77    addjusted wheel radius so that wheel corectly touches ground 
    width: 0.325,
    suspension_max: 0.2,              // fly77   suspension max was too high   adjusted to 0.2
    suspension_min: -0.15,
    suspension_stiffness: 20.0,
    damping_compression: 0.15,
    damping_relaxation: 0.15,
    slip: 1.8,
    roll_influnce: 0.1,
}
 
  this.add_wheel('WHEEL_FL', wheelparam);
  this.add_wheel('WHEEL_FR', wheelparam);
  this.add_wheel('WHEEL_RL', wheelparam);
  this.add_wheel('WHEEL_RR', wheelparam);


  swheel_id = this.get_geomob(0).get_joint("STEERING_WHEEL");
  
  //inputs
  this.register_switch("car/lights/passing", function(v) {
    if(v>0) this.autolights = false;
    this.light_mask(0xf, v>0);
  });
  this.register_switch("car/lights/cabin", function(v) { this.light(lcabin, v>0); });

  //sounds
  

  
  snd1 = this.load_sound("diesel-engine-start.ogg");  // fly77    distinguish sound 1 and sound 2 to bette rhandle them independently 
  snd2 = this.load_sound("diesel-engine-idle.ogg");   // fly77   find info here http://xtrac.outerraworld.com/wiki/sndgrp  or here               https://github.com/Outerra/anteworld/blob/master/include/ot/sndgrp.h
  
 
  //lights
  var fp = {size:0.1, angle:54, edge:0.08, fadeout:0.2, range:50};
  this.add_spot_light({x:0.46783,y:2.24511,z:0.98387}, {y:1}, fp);
  this.add_spot_light({x:-0.46783,y:2.24523,z:0.98387}, {y:1}, fp);

  //tail lights
  fp = {size:0.12, angle:54, edge:0.8, intensity:0.1, range:5, color:{x:1,y:1e-4}, fadeout:0.2};
  this.add_spot_light({x:-0.85767,y:-2.30059,z:1.03871}, {y:-1}, fp);
  this.add_spot_light({x:0.85767,y:-2.30059,z:1.03871}, {y:-1}, fp);
  
  //brakes
  fp = {size:0.12, angle:54, edge:0.8, intensity:0.1, range:5, color:{x:1,y:1e-3}, fadeout:0};
  var brake =
  this.add_spot_light({x:-0.41351,y:-2.41632,z:1.06622}, {y:-1}, fp);
  fp.size = 0.05;
  this.add_spot_light({x:0.46348,y:-2.41632,z:1.06622}, {y:-1}, fp);
  this.add_spot_light({x:0.02469,y:0.10533,z:0.79881}, {y:-1}, fp);
  brakemask = 0xf<<brake;

  //reverse
  fp = {size:0.12, angle:84, edge:0.8, intensity:0.2, range:5, fadeout:0};
  var rev =
  this.add_spot_light({x:-0.24984,y:-0.78261,z:0.95849}, {y:-1}, fp);
  this.add_spot_light({x:0.52854,y:-2.41173,z:0.99484}, {y:-1}, fp);
  revmask = 3<<rev;
  
  //cabin light
  fp = {size:0.05, angle:150, edge:1, range:3, intensity:2, color:{x:1,y:0.8,z:0.7}, fadeout:0.4};
  lcabin = this.add_spot_light({x:0.0,y:-0.01258,z:1.77928}, {z:-1}, fp);
  
  return {mass:1120, steering:3.0, steering_ecf:60, centering: 3.0, centering_ecf: 40, com:{z:-1.6}};
}

//invoked for each new instance of the vehicle
function init_vehicle()
{
  this.set_fps_camera_pos({x:-0.67719,y:0.04902,z:1.5662});	
  this.snd = this.sound();
  this.geom = this.get_geomob(0);
  
  this.sound_source1 = this.snd.create_source("EXHAUST");   // fly77  distinguished sound idle and sound startup to let handle them better independently
  this.sound_source2 = this.snd.create_source("EXHAUST");   // fly77  
  this.snd.set_gain(this.sound_source1 , 1);             // fly77 
  this.snd.set_gain(this.sound_source2 , 0.7);             // fly77    decreased a bit the idle sound volume to better match the startup sound volume
  this.snd.set_ref_distance(this.sound_source1, 9);      // fly77
  this.snd.set_ref_distance(this.sound_source2, 9);      // fly77
 
  this.started = 0;
}

function engine(start)
{
  if(start) {
    this.started=1;
    this.snd.play(this.sound_source1, snd1, false, false);   // fly77  modified to better highlight which sound is handled
  }else {                                                          // fly77
    if (this.started==2){                                           // fly77  if idle sound has been activated then when rpessing again the E button 
      this.snd.stop(this.sound_source2);                          // fly77     switch off sound idle                  
    }                                                              // fly77
  }                                                                 // fly77  
}


const EF = 10000.0;
const BF = 13000.0;
const maxkmh = 154;
const forceloss = EF / (0.4*maxkmh + 2);


//invoked each frame to handle inputs and animate the model
function update_frame(dt, engine, brake, steering)
{
  var kmh = this.speed()*3.6;
  
  //reduce engine force with speed (hack)
  var redux = engine>=0 ? 0.2 : 0.6;
  var esign = engine<0 ? -1 : 1;
  engine = EF*Math.abs(engine);
  var force = (esign>0) == (kmh>=0)
        ? engine/(redux*Math.abs(kmh) + 1)
        : engine;
  force -= forceloss;
  force = Math.max(0.0, Math.min(force, engine));
  engine = esign*force;


  steering *= 0.7;
  this.steer(0, steering);
  this.steer(1, steering);
  
   steerAngle = steering *= 10.05;
	this.geom.rotate_joint_orig(swheel_id, steerAngle, {x:0,y:0,z:0.62});   // fly77  corrected direction of steering wheel axis

  if(this.started>1)
    this.wheel_force(0, engine);
    this.wheel_force(-1, engine);
  
  brake *= BF;
  this.wheel_brake(-1, brake);

  if(this.started==1 && !this.snd.is_playing(snd1)) {          // fly77  corrected to better distinguish which sound is handled
    this.started=2;
    this.snd.play(this.sound_source2, snd2, true, false);      // fly77  corrected to better distinguish which sound is handled
  }
  else if(this.started==2) {
    var pitch = Math.abs(kmh)/20.0;
    var g = kmh>0 ? Math.floor(pitch) : 0;
    var f = pitch - g;
    f += 0.5*g;
    this.snd.set_pitch(this.sound_source2, 0.5*f + 1.0);           // fly77  corrected to better distinguish which sound is handled
  }
  this.animate_wheels();  
}
